ifndef bipedAnim_wdl;
define bipedAnim_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Biped Animation (01)
// class: ANIMATION
// type: SUPPORT
// image: bipedAnim01.pcx
// help: Handle the animation of a biped model.

// needs: bipedPhy01.wdl
// uses:

// prefix: bipedAnim01_
// idcode: 002

// ENDHEADER
// ---------------------------------------------------------------------

// enable: print "debug output" for player on screen?
// id: 1
// define BIPEDANIM01_PLAYER_DEBUG;

// animation
define	_walkswim_dist__002, skill12;   // walk distance . swim distance
define	_runcrawl_dist__002, skill13;   // run distance . crawl distance
define	_standjump_time__002, skill14;  // stand time . jump time
define	_attackduck_time__002, skill15; // attack time . duck time
define	_deathdamage_time__002, skill16;// death time . damage time
define	_runthreshold__002, skill17;	// run threshold

//----------------------------------------------------------------------
// section: Animation Cycle Names

// entry: Stand
// help: Standing animation name
// id: 10
STRING bipedAnim01_stand_str,"stand";
// entry: Walk
// help: Walking animation name
// id: 11
STRING bipedAnim01_walk_str,"walk";
// entry: Run
// help: Running animation name
// id: 12
STRING bipedAnim01_run_str,"run";
// entry: Jump
// help: Jumping animation name
// id: 13
STRING bipedAnim01_jump_str,"jump";
// entry: Duck
// help: Ducking animation name
// id: 14
STRING bipedAnim01_duck_str,"duck";
// entry: Crawl
// help: Crawling animation name
// id: 15
STRING bipedAnim01_crawl_str,"crawl";
// entry: Swim
// help: Swimming animation name
// id: 16
STRING bipedAnim01_swim_str,"swim";
// entry: Wade
// help: Wading animation name
// id: 17
STRING bipedAnim01_wade_str,"walk";
// entry: Death
// help: Death animation name
// id: 18
STRING bipedAnim01_death_str,"death";
// entry: Attack
// help: Attack animation name
// id: 19
STRING bipedAnim01_attack_str,"attack";


//----------------------------------------------------------------------
// section: Misc

// entry: Stillness Threshold
// entry_help: If the movement distance is less than this value, the actor/player
// entry_help:is considered to be 'standing still' as far as the biped animation
// entry_help:functions are concerned (so it will choose to play the stand animations)
// id: 20
var bipedAnim01_still_threshold = 0.025;

//----------------------------------------------------------------------

// pointers to animation strings
string*	bipedAnim01_old_strp;	// pointer to old animation string
string*	bipedAnim01_cur_strp;	// pointer to current animation string

// stand, walk, run, duck, crawl, jump, swim, attack, and death
define	bipedAnim01_mode_quit_const = -1;// quit animation
define	bipedAnim01_mode_none_const = 0;	// no animation
define	bipedAnim01_mode_stand_const = 1;	// standing/idle animation (tick)
define	bipedAnim01_mode_walk_const = 2;	// walking animation (dist)
define	bipedAnim01_mode_run_const = 3;		// running animation (dist)
define	bipedAnim01_mode_wade_const = 4;	// wading animation
define	bipedAnim01_mode_duck_const = 5;	// ducking animation	(?)
define	bipedAnim01_mode_crawl_const = 6;	// crawling animation (?)
define	bipedAnim01_mode_jump_const = 7;	// jumping animation (tick)
define	bipedAnim01_mode_swim_const = 8;	// swimming animation (?)
define	bipedAnim01_mode_attack_const = 9;	// attack animation
define	bipedAnim01_mode_death_const = 10;  // dying animation (* an exit animation)



// Desc: animate a blended transition between two states
//
//  'my' must be set to an actor (i.e. my = player)
//  my._biped01_anim_dist is set to 0 at the end of the transition, it is equal
//to -1 the rest of the time.
//
// str_anim_source is a string containing the source animation name
// str_anim_target is a string containing the target animation name
// trans_ticks is the number of ticks this function will take to complete
//
// e.g. BipedAnim01_Transition("walk","stand",16);
//		- blend from walk to stand in 16 ticks (one second)
function BipedAnim01_Transition(str_anim_source,str_anim_target,trans_ticks)
{
	var	perBlend_total;	// total percent blend (0-none, 100-finished)
	var	perBlend_inc;     // bend increaments
	var	source_frame_percent;	// the source frame % blending from

	source_frame_percent = my._biped01_anim_dist;
	my._biped01_anim_dist = -1; // used to flag the calling function

 	perBlend_total = 0;
	perBlend_inc = 100/trans_ticks;

	while(perBlend_total < 100)
	{
		// blend to the start of the target cycle
  		//--old ent_cycle(str_anim_source,source_frame_percent);
		ent_animate(me,str_anim_source,source_frame_percent,ANM_CYCLE);
		ent_blend(str_anim_target,0,perBlend_total);
		wait(1);
		perBlend_total += max(0.001,perBlend_inc*TIME);	// make sure we get some transition even if fps is extream
	}

	// set to start of animation cycle
	my._biped01_anim_dist = 0;                       // done with animation
	//--old ent_cycle(str_anim_target,my._biped01_anim_dist);
	ent_animate(me,str_anim_target,my._biped01_anim_dist,ANM_CYCLE);
}







// Desc: get the animation string for the current animatino mode
//
// calls:
// called by: actor_adv_anim2
// uses: MY._standjump_time__002, MY._ADVANIM_DIST (read only)
//
//
// set bipedAnim01_cur_strp to the right string
// return the cycle value if quant based or negitive cycle if tick based
function BipedAnim01_Set_String(anim_mode)
{
	// handle standing animation (tick based)
	if(anim_mode == bipedAnim01_mode_stand_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_stand_str;
		return( -INT(my._standjump_time__002) * 4);	// stand ticks
	}

	// handle walking animation (quant based)
	if(anim_mode == bipedAnim01_mode_walk_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_walk_str;
		return( INT(my._walkswim_dist__002) * 0.25);	// walk dist
	}

	// handle running animation (quant based)
	if(anim_mode == bipedAnim01_mode_run_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_run_str;
 		return( INT(my._runcrawl_dist__002) * 0.25);	// run dist
	}

	// handle wading animation (quant based)
	if(anim_mode == bipedAnim01_mode_wade_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_wade_str;
		// NOTE! uses same distance as walking!
		return( INT(my._walkswim_dist__002) * 0.25);	// wade (walk) dist
	}

	// handle jumping animation (tick based)
	if(anim_mode == bipedAnim01_mode_jump_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_jump_str;
		return( -FRC(my._standjump_time__002) * 400);	// jump ticks
	}

	// handle swimming animation (quant based)
	if(anim_mode == bipedAnim01_mode_swim_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_swim_str;
 		return( FRC(my._walkswim_dist__002) * 25);	// swim dist
	}

	// handle ducking animation (tick based)
	if(anim_mode == bipedAnim01_mode_duck_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_duck_str;

  		return( -FRC(my._attackduck_time__002) * 400);	// duck ticks
	}

	// handle crawling animation (quant based)
	if(anim_mode == bipedAnim01_mode_crawl_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_crawl_str;
 		return( FRC(my._runcrawl_dist__002) * 25);	// crawl dist
	}

	// handle attack animation (tick based)
	if(anim_mode == bipedAnim01_mode_attack_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_attack_str;
  		return( -INT(my._attackduck_time__002) * 4);	// attack ticks
	}

	// handle dying animation (tick based)
	if(anim_mode == bipedAnim01_mode_death_const)
	{
		bipedAnim01_cur_strp = bipedAnim01_death_str;
  		return( -INT(my._deathdamage_time__002) * 4);	// death ticks
	}


	// ..add new animation modes here

	// default case
	bipedAnim01_cur_strp = bipedAnim01_stand_str;
	return(16);
}



// Desc: evaluate the entity and select the correct animation mode
//
// calls:
// called by: actor_adv_anim2
//
//	takes: cur_anim_mode - current animation mode
//	uses:^my._FIREMODE  - see if actor is shooting
//		  ^my._MOVEMODE  - find out what movement state actor is in
//		  ^my._biped01_dist  - how far (quants) the actor moved last frame
//		  ^MY._runthreshold__002 - to calculate walk/run distance
//		  ^MY._health__003 - see if biped is dead/dying
//			temp
//
// returns the animation mode
// sets temp to 1 if this animation is 'once only' (e.g. jump)
// sets temp to -1 if this animation doesn't use blending and is 'once only' (fire)
//
function	BipedAnim01_Get_Mode(cur_anim_mode)
{

// =====================================================================
// _MODE_STILL means "no animation"
// =====================================================================
	if(MY._biped01_state == biped01_state_none_const)
	{
		temp = 0; // continuous animation cycle
		return(bipedAnim01_mode_none_const);
	}

// =====================================================================
// special case animations that 'over ride' all other animation states
// =====================================================================

	// death
	if(MY._health__003 <= 0)
	{
		temp = 1;	// "once only" flag
		return(bipedAnim01_mode_death_const);
	}


	// attack mode
	if(MY._biped01_state2_mod == biped01_state2_mod_attack_const)
	{
		temp = -2;	// "once only", no blend, do full cycle flag
		return(bipedAnim01_mode_attack_const);
	}


// =====================================================================
// movemode animation (after special case)
// =====================================================================

	// walking, running, or standing mode
	if(my._biped01_state == biped01_state_walk_const)
	{
		// mod: 06/07/02 - added time correction to distance calculations
		// test for stand, walk, or run
		temp = abs(my._biped01_dist/TIME);   // absolute time corrected distance (quants)

		// movement less than stillness threshold?
		if(temp < bipedAnim01_still_threshold)
		{
			// standing still
			temp = 0; // continuous animation cycle
			return(bipedAnim01_mode_stand_const);
		}

		// NOTE: we check to see if we are running already so we can have
		//an overlapping 'dead zone' where the animation remains the same
		//so we can avoid a constant state 'flip-flop'

		// decide whether to play the walk or run animation
		if(cur_anim_mode == bipedAnim01_mode_run_const)	// we are in running mode now
		{
			if(temp < ((my._runthreshold__002)*0.75))	// fall back to walking      0.75
			{
				temp = 0; // continuous animation cycle
				return(bipedAnim01_mode_walk_const);
			}
			temp = 0; // continuous animation cycle
			return(bipedAnim01_mode_run_const);	// continue running
		}


		if(temp < ((my._runthreshold__002)*1.15))	// keep walking      1.15
		{
			temp = 0; // continuous animation cycle
			return(bipedAnim01_mode_walk_const);
		}
		// start running
		temp = 0; // continuous animation cycle
		return(bipedAnim01_mode_run_const);

	}

	if(MY._biped01_state == biped01_state_wade_const)
	{
		// test for stand or wade
		temp = abs(my._biped01_dist/TIME);

		// movement less than stillness threshold?
		if(temp < bipedAnim01_still_threshold)
		{
			// standing still
			temp = 0; // continuous animation cycle
			return(bipedAnim01_mode_stand_const);
		}
		// wading animation
		temp = 0; // continuous animation cycle
		return(bipedAnim01_mode_wade_const);
	}

	if( MY._biped01_state == biped01_state_jump_const)
	{
		temp = 1;	// "once only" flag
		return(bipedAnim01_mode_jump_const);
	}

	if( MY._biped01_state == biped01_state_swim_const)
	{
		temp = 0; // continuous animation cycle
		return(bipedAnim01_mode_swim_const);
	}

	if( MY._biped01_state == biped01_state_duck_const)
	{
		// test for duck or crawl
		temp = abs(my._biped01_dist/TIME);
		// movement less than stillness threshold?
		if(temp < bipedAnim01_still_threshold)
		{
			temp = 0; // continuous animation cycle
			return(bipedAnim01_mode_duck_const);
		}
		temp = 0; // continuous animation cycle
		return(bipedAnim01_mode_crawl_const);

	}

	if( MY._biped01_state == biped01_state_crawl_const)
	{
		temp = 0; // continuous animation cycle
		return(bipedAnim01_mode_crawl_const);
	}

	// ..add new movemode animation modes here

	// default case
	// breakpoint; // uncomment to catch animations that 'fall thru'
	temp = 0; // continuous animation cycle
	return(bipedAnim01_mode_stand_const);	// when all else fails, stand around
}

// Desc: Animate biped model (without bones)
//		- uses transitions
//		- loops every frame
//
//   'my' must be set before calling
//		my._biped01_dist must be set to the distance that the actor has covered (in quants)
//		my._biped01_anim_dist is modified in this function
//
//    global bipedAnim01_cur_strp is used
//
function BipedAnim01_Update_Loop()
{
	//  NOTE! We can NOT use proc_late with this function since other functions
	// like attached weapon animation depend on this function running first.
	// This means that animation may be off by one frame compared to actual
	// player movement but the effect is not noticeable due to the use of
	// animation blending.
//	proc_late();	// used to delay until end of frame (set only once!)

	// init local variables
	var	cur_anim_mode;		// the current animation mode
	var	old_anim_mode;		// the old animation mode

	var	anim_cycle_val;	// the value of a complete animation cycle (in ticks or distance)
	var	anim_percent;		// percent value (0-100) of current animation cycle

	var	qOnceOnly;		// flag, equals 1 if this is a one cycle only animation
								//       equals -1 if this is once only plus no blend
								//       equals -2 if this is once only, plus no blend, plus force full cycle (time only)
								//			equals 0 if cycles

	while(my != null)	// loop while we have an actor
	{
		// check movement/fire mode to set current animation mode and anim_cycle_val
		cur_anim_mode = BipedAnim01_Get_Mode(cur_anim_mode);
		qOnceOnly = temp;	// set 'once only' flag (temp set in BipedAnim01_Get_Mode())

		if(cur_anim_mode == bipedAnim01_mode_none_const) { wait(1); continue; }	// no animation
		if((qOnceOnly != 0) && (my._biped01_anim_dist == -99)) { wait(1); continue; }	// no animation

		if(qOnceOnly == -2)
		{
			// force a full animation loop of the cur_anim_mode

			my._biped01_anim_dist = 0.125;	// greater than 0 to avoid div zero error later.

			// NOTE: only time based animations are valid
			anim_cycle_val = -BipedAnim01_Set_String(cur_anim_mode);
			if(anim_cycle_val == 0) { anim_cycle_val = 16; diag("\nWARNING! BipedAnim01::nBipedAnim01_Update_Loop()- Bad value in anim_cycle_val (replaced with 16)");}

			while(my._biped01_anim_dist < anim_cycle_val)
			{
  				my._biped01_anim_dist += time;          // update time
 		  		anim_percent =  (100 * my._biped01_anim_dist / anim_cycle_val);
				BipedAnim01_Set_String(cur_anim_mode);	// NOTE: bipedAnim01_cur_strp is a global string, must be set each frame (until we get local strings)
 				ent_animate(me,bipedAnim01_cur_strp,anim_percent,0);

				wait(1);
			}
			continue;
		}

		// if current animation mode is not the same as old animation mode
		if((cur_anim_mode != old_anim_mode) && (qOnceOnly > -1))
		{
			// get old and new values
			BipedAnim01_Set_String(old_anim_mode); // get the old animation mode string
			bipedAnim01_old_strp = bipedAnim01_cur_strp;	// save old value
			anim_cycle_val = BipedAnim01_Set_String(cur_anim_mode); // get new value
			if(anim_cycle_val == 0) { anim_cycle_val = 16; diag("\nWARNING! BipedAnim01::nBipedAnim01_Update_Loop()- Bad value in anim_cycle_val (replaced with 16)");}

 			// set my._biped01_anim_dist to percentage
			MY._biped01_anim_dist = anim_percent;

			// do transition (continue in this loop until transition is complete)
			BipedAnim01_Transition(bipedAnim01_old_strp,bipedAnim01_cur_strp,2);

			// wait here until transition is done
			while(my._biped01_anim_dist == -1)
			{
ifdef BIPEDANIM01_PLAYER_DEBUG;
if(my._gid01_id == gid01_player_const)
{
draw_text(bipedAnim01_old_strp,200,10,vector(100,100,255));
draw_text("to",200,30,vector(100,100,255));
draw_text(bipedAnim01_cur_strp,200,50,vector(100,100,255));
}
endif;
				wait(1);
			}
		}

		// old animation mode = current animation mode
		old_anim_mode = cur_anim_mode;

		// get animation string for current animation mode
		// NOTE: move this to 'transition' section when we get local strings
		anim_cycle_val = BipedAnim01_Set_String(cur_anim_mode);
		if(anim_cycle_val == 0) { anim_cycle_val = 16; diag("\nWARNING! BipedAnim01::nBipedAnim01_Update_Loop()- Bad value in anim_cycle_val (replaced with 16)");}

		// INC MY._biped01_anim_dist by TIME or my._biped01_dist (depending on animation)
		// if tick based
		if(anim_cycle_val < 0)
		{
 	  		my._biped01_anim_dist += time;          // update time
			anim_cycle_val = -anim_cycle_val;	// make it positive
		}
		else	// else dist based
		{
			// set the percent_width covered using percent of the model width
  			//my._biped01_dist is set in the actor's movement functions

			my._biped01_anim_dist += (MY._biped01_dist / (MY.MAX_X-MY.MIN_X));
		}

		// wrap animation time to a value between zero and anim_stand_ticks
		//and check to see if this cycle it a "once only",
		if(MY._biped01_anim_dist >= anim_cycle_val)
		{
			if(qOnceOnly != 0)   // a 'once only' animation
			{
				// are we finishing the death animation?
				if(cur_anim_mode == bipedAnim01_mode_death_const)
				{
					return;	// exit function
				}

				// finished "once only" animation, switch to default
				my._biped01_state = biped01_state_walk_const;	// switch to default
				MY._biped01_state2_mod = biped01_state2_mod_none_const;	// reset any state2
				my._biped01_anim_dist = 0;


				wait(1);   // must wait a frame to allow calling function time to respond
				continue;
	 		}

			while(MY._biped01_anim_dist > anim_cycle_val)    // 'wrap' time/dist
			{
				MY._biped01_anim_dist -= anim_cycle_val;
			}
		}

		// warp for negitive animation
		while(MY._biped01_anim_dist < 0)
		{
			MY._biped01_anim_dist += anim_cycle_val;
		}
		// calculate a percentage out of the animation time/dist
		if(anim_cycle_val == 0) {beep; anim_cycle_val = 1;  } // error!
		anim_percent =  (100 * my._biped01_anim_dist / anim_cycle_val);

		// check if it is a non-looping once only animation
		if(qOnceOnly != 0)
		{
			// animate in current mode (using time step/distance traveled)
			//-- old ent_frame(bipedAnim01_cur_strp,anim_percent);
			ent_animate(me,bipedAnim01_cur_strp,anim_percent,0);
		}
		else
		{
			// animate in current mode (using time step/distance traveled)
			//-- old ent_cycle(bipedAnim01_cur_strp,anim_percent);
			ent_animate(me,bipedAnim01_cur_strp,anim_percent,ANM_CYCLE);
		}
ifdef BIPEDANIM01_PLAYER_DEBUG;
if(my._gid01_id == gid01_player_const)
{
draw_text(bipedAnim01_cur_strp,200,10,vector(100,100,255));
}
endif;


		wait(1);	// wait a frame
	} // end "while(my != null)" loop


}


endif;