ifndef bipedPhy_wdl;
define bipedPhy_wdl;



// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Biped Physics (01)
// class: MOVEMENT
// type: SUPPORT
// image: bipedPhy01.pcx
// help: Handle the physics of motion for bipeds in the world.
//
// idcode: 033
//
// ENDHEADER
// ---------------------------------------------------------------------



// enable: Debug?
// help: Display debug information?
//define	BIPEDPHY01_DEBUG;


ifdef	BIPEDPHY01_DEBUG;
var bipedPhy01_debug_data[20];

font bipedPhy01_debug_font = <ackfont.pcx>,6,9;
text bipedPhy01_debug_txt
{
     font = bipedPhy01_debug_font;
     pos_x = 430;
     pos_y = 1;
     string = "Scan Results (C_Collide)\n============\n->height(*10)\n->trace_hit\n->state\n\n\n\n->normal.x(x100)\n->normal.y(x100)\n->normal.z(x100)\n->you\n->state\n->dist(*100)\n->dist/TIME(*100)\n->s_x\n->s_y\n->s_z\n->vel_x\n->vel_y\n->vel_z";
     flags = d3d, visible;
}


panel bipedPhy01_debug_pan
{
     pos_x = 400;
     pos_y = 10;
     digits = 0, 9,  5, bipedPhy01_debug_font, 10, bipedPhy01_debug_data[0];  //   height
     digits = 0, 18, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[1];			// trace_hit
     digits = 0, 27, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[2]; 		// state
     digits = 0, 36, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[3]; //
     digits = 0, 45, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[4]; //

     digits = 0, 63, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[5]; // normal.x
     digits = 0, 72, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[6]; // normal.y
     digits = 0, 81, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[7]; // normal.z
     digits = 0, 90, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[8]; // you
     digits = 0, 99, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[9]; //
     digits = 0, 108, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[10]; //dist
     digits = 0, 117, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[11];   // dist / time
     digits = 0, 126, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[12];
     digits = 0, 135, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[13];
     digits = 0, 144, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[14];
     digits = 0, 153, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[15];
     digits = 0, 162, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[16];
     digits = 0, 171, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[17];
     digits = 0, 180, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[18];
     digits = 0, 189, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[19];
     flags =  d3d, visible;
}
endif;

// Used for Performance report (collision)
//define	bipedPhy01_perform_report;
ifdef	bipedPhy01_perform_report;

var bipedPhy01_scan_elapsed;
var bipedPhy01_move_elapsed;
var bipedPhy01_calls;
var bipedPhy01_move_calls;
string bipedPhy01_perform_str[60];

starter BipedPhy01_Perform
{
	while(1)
	{
		diag("\nBIPEDPHY Calls: ");
		str_for_num(bipedPhy01_perform_str,bipedPhy01_calls);
		diag(bipedPhy01_perform_str);

		diag(" Total t: ");
		str_for_num(bipedPhy01_perform_str,time_actions);
		diag(bipedPhy01_perform_str);

		diag(" Move t: ");
		str_for_num(bipedPhy01_perform_str,bipedPhy01_move_elapsed/1000);	// convert from microsec to ms
		diag(bipedPhy01_perform_str);
		if(time_actions > 0)   // avoid div by zero
		{
			diag(" (");
			str_for_num(bipedPhy01_perform_str,(bipedPhy01_move_elapsed/time_actions)/10);
			diag(bipedPhy01_perform_str);
			diag("%)");
		}

 		diag(" SCAN t: ");
		str_for_num(bipedPhy01_perform_str,bipedPhy01_scan_elapsed/1000);	// convert from microsec to ms
		diag(bipedPhy01_perform_str);
		if(time_actions > 0)   // avoid div by zero
		{
			diag(" (");
			str_for_num(bipedPhy01_perform_str,(bipedPhy01_scan_elapsed/time_actions)/10);
			diag(bipedPhy01_perform_str);
			diag("%)");
		}


		sleep(1);
		bipedPhy01_calls = 0;	// reset counters
		bipedPhy01_move_calls = 0;
		bipedPhy01_move_elapsed = 0;	// reset timer
		bipedPhy01_scan_elapsed = 0;	// reset timer
		wait(1);
	}


}
endif;


// =====================================================================
// USER DEFINED VALUES

// section: Friction

// entry: "Glide"
// help: Friction for gliding along surfaces (0..1)
// id: 10
var	move_friction = 0.25;

// entry: Ground
// help: Friction while on solid ground
// id: 11
var	bipedPhy01_grnd_friction = 0.5;

// entry: Ground Angular
// help: Effects panning on ground
// id: 12
var	bipedPhy01_grnd_ang_fric = 1.0;

// entry: Air
// help: Friction used when jumping/falling.
// id: 13
var	bipedPhy01_air_friction = 0.1;

// entry: Water
// help: Friction used while swimming
// id: 14
var	bipedPhy01_water_friction = 1.65;
// entry: Water Angular
// help: Effects panning in water
// id: 15
var	bipedPhy01_water_ang_friction = 1.65;


//---------------------------------------------------
// section: Environmental Forces


// entry: Float force
// help: this is the amount of upward force applied to bipeds in a passible surface.
// id: 16
var	bipedPhy01_water_float = 1.85;


// entry: Gravity
// help: Downward pull on bipeds (keeps them from floating away)
// id: 17
var	bipedPhy01_gravity = 9;


//---------------------------------------------------
// section: Slope values

// entry: Slope factor
// help: If the Z normal is less then this, the biped will slide down (no climb)
// id: 18
var	bipedPhy01_slope_fac = 0.75;





//----------------------------------------------------------------------
// section: Damage from Falling

// entry: Safe Time
// help: Maximum time biped can fall without damage (ticks)
// id: 19
define bipedPhy01_safe_fall_time_const = 9;

// entry: Damage Const
// help: Constant used to calculate damage from falling
// id: 20
define bipedPhy01_fall_damage_const = 1.75;

//----------------------------------------------------------------------
// section: Damage from drowning

// entry: Safe Time
// help: Time underwater before taking damage (ticks)
// id: 21
define	bipedPhy01_safe_underwater_time_const = 220;

// entry: Damage
// help: Damage per tick while drowning
// id: 22
define	bipedPhy01_drowning_damage_const = 0.125;


///////////////////////////////////////////////////////////////////////
// Entity skill & flag definitions

define	_health__003,SKILL1;	// health of the biped
define	_armor__003, SKILL2;	// armor protects from some sorts of damage


// max force vectors
define _max_force_x__003,SKILL3;
define _max_force_y__003,SKILL4;
define _max_ang_force_pan__003,SKILL6;
//DEFINE _biped01_max_ang_force_tilt,SKILL7;
//DEFINE _biped01_max_ang_force_roll,SKILL8;

define _trigger_range__003, SKILL7;    // distace that things are triggered

define _force_mult_003,SKILL8;

// height limits
//define _step_height__003,SKILL9;   // REPLACED with _move_min_z__003
define _move_min_z__003,SKILL9;       // used to set the move_min_z durring moves
define _jump_height__003,SKILL10;
define _duck_height__003,SKILL11;





// velocity vectors
/*
// Velocity in quants/tick
define	_gid01_vel_x, skill21;        // current velocity (quants/tick)
define	_gid01_vel_y, skill22;        // current velocity (quants/tick)
define	_gid01_vel_z, skill23;        // current velocity (quants/tick)


// ang velocity in deg/tick
define _gid01_ang_vel_pan, SKILL24;
define _gid01_ang_vel_tilt, SKILL25;
define _gid01_ang_vel_roll, SKILL26;
*/
define _biped01_dist, SKILL27;  // distance covered last frame

define _biped01_state, skill28;	// biped state:
define biped01_state_none_const,0;		// no movement (i.e. player dead)
//--define biped01_state_stand_const,1;		// standing
define biped01_state_walk_const,2;    	// walking. running, standing
//--define biped01_state_run_const,3;     	// running
define biped01_state_jump_const,4;    	// jumping
define biped01_state_duck_const,5;    	// ducking or crawling
define biped01_state_crawl_const,6;		// crawling (move while ducked)
define biped01_state_swim_const,7;		// swimming
define biped01_state_wade_const,8;		// wading in water
define biped01_state_dying_const,10; 	// dying
define biped01_state_dead_const,11;    // finished dying
define biped01_state_activate_const,12;// press a button or switch

define _biped01_state_mod, skill29;	// biped state modifier:
define biped01_state_mod_none_const,0;					// no modifier
define biped01_state_mod_fall_const,1;			// biped is falling
define biped01_state_mod_underwater_const,2;	// biped is underwater
define biped01_state_mod_damage_const,3;		// biped has taken damage from environment (falling, crushing, etc.)
define biped01_state_mod_drown_const,4;		// player is drowning
define _biped01_state_mod_value, skill30;	// state modifier value, dependent on state modifier

// note: state 2 modifiers can be used in combination with state 1 modifier above
define _biped01_state2_mod, skill34; // biped state modifier 2:
define biped01_state2_mod_none_const,0;	// no modifier
define biped01_state2_mod_attack_const,1;	// preforming an attack
define biped01_state2_mod_damage_const,2;	// taking damage
define _biped01_state2_mod_value, skill35;	// state2 modifier value, dependent on state2 modifier


define _biped01_jump_target, SKILL32;	// target to reach while jumping

define _biped01_anim_dist, SKILL33;    // animation distance



define _no_fall_dam__003, FLAG1;			// if set, entity does not take damage from falls



//----------------------------------------------------------------------
// "Local" global values (used for parameter passing)
var	bipedPhy01_height;						// height from the bottom of the model to the nearest solid surface.
//var	bipedPhy01_passable_height;			// height from the bottom of the model to the nearest passable surface. (0 if none)
var	bipedPhy01_surface_normal_vec[3];	// surface normal of the surface beneight the ent
var	bipedPhy01_surface_vel_vec[3];		// the velocity of the surface under the ent

entity* bipedPhy01_temp_ent;




// =====================================================================
// MISC FUNCTIONS

// Calculate the damage from a fall and return that value
function	BipedPhy01_Fall_Damage()
{
	if(my._no_fall_dam__003 == 1) { return(0); }

	// calculate damage depending on fall time
 	return(10 + INT(((my._biped01_state_mod_value - bipedPhy01_safe_fall_time_const) * bipedPhy01_fall_damage_const)));
}



ifndef GID01_OLD_COLLIDE;
// =====================================================================
// ======== USING THE NEW COLLISION CODE ===============================
// =====================================================================


// =======================================================================
// C_COLLIDE CODE!

// =====================================================================
// ROTATION (PAN) FUNCTIONS

// Desc: handle biped panning (old, not used anymore)
//
// Input: ang_force (vec) set to the panning force
//	       ang_fric is set to whatever friction
// Assume: my is set to a biped01 entity,
function BipedPhy01_Pan_Force(pan_force,ang_fric)
{
	var	delta_ang_fric;

	delta_ang_fric = max((1-ang_fric * time),0);

	my._gid01_ang_vel_pan = (TIME * pan_force) + (delta_ang_fric * my._gid01_ang_vel_pan);
	temp.pan += my._gid01_ang_vel_pan * TIME;
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + glide + use_box;
	c_rotate(my,vector((my._gid01_ang_vel_pan * TIME),0,0),move_mode);
}


// Desc: handle biped panning with absolute values (not time correct)
//
// Input: ang_dist (vec) set to the panning amount (modified)
//
// Result:
//		Sets: my._gid01_ang_vel_pan equal to the angular velocity
//				ang_dist is set to the actual distace moved this frame
//		The entity 'my' is rotated using c_rotate().
// Assume: my is set to a biped01 entity,
function BipedPhy01_Pan(ang_dist)
{

//	ang_fric = 1/(min(1,ang_fric));
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + glide + use_box;
	ang_dist = c_rotate(my,vector(ang_dist,0,0),move_mode);
	my._gid01_ang_vel_pan = ang_dist / TIME; // calculate ang vel for use later
}












//==============================================================================================================================================

// desc: simple scan. Does NOT detect passable blocks (water,forcefields, etc.)
//
// Assume: my is set to a valid biped01 entity (see doc for valid skills)
var bipedPhy01_to_vec[3];   // can't use local vecs when calling functions
var bipedPhy01_from_vec[3];
function	BipedPhy01_Scan()  // c_collide version
{
if(my == NULL) { diag("************NULLED MY IN BipedPhy01_Scan****"); }

 	// Use trace to scan the surface under the entity
	vec_set(bipedPhy01_from_vec,my.x);
	vec_set(bipedPhy01_to_vec,my.x);
	trace_mode = IGNORE_ME + IGNORE_PASSABLE + IGNORE_PASSENTS + ACTIVATE_SONAR;  // + USE_BOX
	bipedPhy01_to_vec.z += 2*my.min_z - 6;//-= (160 - my.min_z);//+= (my.min_z - 15); //-= 4000;
	bipedPhy01_height = c_trace(bipedPhy01_from_vec,bipedPhy01_to_vec,trace_mode);
	// height is distance from center minus the min_z and step_height
	bipedPhy01_height += my.min_z;

	// Use the new "trace_hit" flag to see if we hit anything
	// NOTE: Use with new collision code!
	if(0 == trace_hit) // 0-if nothing hit
	{
		bipedPhy01_height = -2*my.min_z + 6;	// greater than x units off the ground (in air)
	}


	//-------------------------------------------------------------------
	// Trace 16 units down using a ray. Ignore me, sprites, and passents.
	// Activate sonar.
	//
	// Results:
	//  	-bipedPhy01_height: height above surface or 16 (if no surface hit)
	//		-trace_hit: 1-if surface hit, 0 otherwise
	//-------------------------------------------------------------------

	bipedPhy01_temp_ent = you;	// not on or in a passable block/entity

	vec_set(bipedPhy01_surface_normal_vec.x,NORMAL.X); // normal is set in trace

	// if the player is standing on a platform or in a forcefield,
	if(bipedPhy01_temp_ent != NULL)
	{
		if(gid01_platform_const == bipedPhy01_temp_ent._gid01_id)
		{
			// add current platform velocity
			vec_set(bipedPhy01_surface_vel_vec[0],bipedPhy01_temp_ent._gid01_vel_x);
		}
		else
		{
			// no added velocity due to platform
			vec_set(bipedPhy01_surface_vel_vec[0],nullvector);
		}
	}
	else
	{
		// no added velocity
		vec_set(bipedPhy01_surface_vel_vec[0],nullvector);
	}

	return(0);
}



// Desc: move on the ground, air, or wading
//
// assume: my is set to a biped01
//			  bipedPhy01_height is set to the current height above a surface
//			  bipedPhy01_grnd_friction is set to the ground friction value
function	BipedPhy01_Move_Ground(&force_vec, pan_dist)
{

	var	friction;	// friction constant
	var	delta_fric; // friction over time value
	var	dist[3];		// distance covered this frame in direction faced
	var	abs_dist[3];// additional distance covered this frame in world space
	var	abs_force[3];//forces from the environment (gravity, elevators, etc.)
	var	q_jump_or_duck; // 1: jump, -1: duck, 0: neither
	var	old_loc[3];	//  old location

	// jump or duck?
	q_jump_or_duck = sign(force_vec[2]);
	force_vec[2] = 0;

	// this biped doesn't tilt or roll
	my.tilt = 0;
	my.roll = 0;

	// rotate the player (absolute)
	BipedPhy01_Pan(pan_dist);

	///////////////////////////////////////////
	// drop to ground and check for collision


	if(bipedPhy01_height<0) // in the ground?
	{
		// push out of ground

		// note: using force to slowly push the biped out of the ground, while
		//   technically better, was considered "poor collision detection" by
		//   most users. Most users would prefer a "pop" in movement over having
		//   the biped model penetrate the ground.
		abs_dist.z = -bipedPhy01_height;
	}
	else   // gravity is in effect.
	{
		abs_dist.z = accelerate(my._gid01_vel_z,-bipedPhy01_gravity,0); 						// accel downward
	}

	// do the move...
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box;
	if((bipedPhy01_height > -my.min_z) // we are resting on an edge/slope higher then half our height, or in the air
		|| ( bipedPhy01_surface_normal_vec.z < bipedPhy01_slope_fac)) // slope is too steep...
	{
		move_mode += glide; // glide down slopes and tall edges
	}
	c_move(my,nullvector,vector(0,0,abs_dist.z),move_mode);


   // Use the new "trace_hit" flag to see if we hit anything
	if(0 == trace_hit) // 0-if nothing hit
	{
ifdef	BIPEDPHY01_DEBUG;
Draw_Text("AIR",50,80,vector(255,0,0));
endif;
		bipedPhy01_height = 16;	// Still in the air

		friction = bipedPhy01_air_friction; // use air friction

   	// airborne - reduce all relative forces
		// to prevent him from jumping or further moving in the air
		force_vec[0] *= 0.2; // don't set the force completely to zero, otherwise
		force_vec[1] *= 0.2; // player could be stuck on top of a non-wmb entity
		force_vec[2] = 0;

		vec_set(abs_force,nullvector);  // gravity is handled above.


 		// only falling if moving downward (but not if ducking)
		if( (my._gid01_vel_z < 0))//?? && ((MY._MOVEMODE != _MODE_DUCKING) && (MY._MOVEMODE != _MODE_CRAWLING)) )
		{
			// check to make sure we are not in another state
			if(my._biped01_state_mod != biped01_state_mod_fall_const)
			{
				my._biped01_state_mod = biped01_state_mod_fall_const; // set falling flag
				my._biped01_state_mod_value = 0; // reset falling time
			}
			my._biped01_state_mod_value += TIME;   // add falling time
		}


	}
	else    // on the Ground
	{
		bipedPhy01_height = 0;
		friction = bipedPhy01_grnd_friction; // use ground friction

		//-------------------------------------------------------------------------------------------------------------------------
		// Calculate falling damage
 		if((my._biped01_state_mod == biped01_state_mod_fall_const) &&  // we are falling
			(my._biped01_state_mod_value > bipedPhy01_safe_fall_time_const))	// we've been falling more then the safe amount of time
  		{
			// indicate damage from environment
			my._biped01_state2_mod = biped01_state_mod_damage_const;
			my._biped01_state2_mod_value = BipedPhy01_Fall_Damage();		// take damage depending on fall_time
			my._biped01_state_mod = biped01_state_mod_none_const; // reset flag (we are on ground)
			my._biped01_state_mod_value = 0;
		}
		else
		{
			// note: this works ONLY because no other state can exist at this point
			//(e.g. the player is not underwater and not falling)
			my._biped01_state_mod = biped01_state_mod_none_const; // reset flag (we are on ground)
			my._biped01_state_mod_value = 0;
		}
		//-------------------------------------------------------------------------------------------------------------------------

 		// reset absolute forces
		vec_set(abs_force,vector(0,0,0));




		// do we wish to jump?
		if((q_jump_or_duck>0) && (my._biped01_jump_target <= 0)    // no jumping from ducking, or if we are already jumping
			&& (bipedPhy01_surface_normal_vec.z >= bipedPhy01_slope_fac) && (my._biped01_state != biped01_state_wade_const)) // no jumping from slopes or water
		{
			// start a jump
			my._biped01_jump_target = my._jump_height__003; // jump delta
			my._biped01_state = biped01_state_jump_const; // set state to jump
		}
		else
		{
			// walking/standing/running on solid ground
			my._biped01_state = biped01_state_walk_const;
		}



	} // end "on the ground"


	// calculate velocities
	dist.x = accelerate(my._gid01_vel_x,force_vec[0],friction);
	dist.y = accelerate(my._gid01_vel_y,force_vec[1],friction);
	dist.z = 0;                      // dz = 0  (only gravity and jumping)


	if(my._biped01_jump_target <= 0)	// not jumping
	{
	  //	my._gid01_vel_z = (TIME * abs_force.z) + (delta_fric * my._gid01_vel_z);    // gravity force (old way)
		abs_dist.z = accelerate(my._gid01_vel_z,abs_force.z,friction);	// new way
	}
	else	// jumping
	{
	 	my._gid01_vel_z = sqrt(my._biped01_jump_target*2*bipedPhy01_gravity);    // jumping force
		my._biped01_jump_target -= my._gid01_vel_z * TIME;	// reduce the delta
		abs_dist.z = my._gid01_vel_z * TIME;
	}


	// calculate absolute distance to move
	// NOTE:
	//  - bipedPhy01_surface_vel_vec is any velocity added from standing on a moving platform
	abs_dist.x = ((abs_force.x * time) + bipedPhy01_surface_vel_vec[0]) * TIME;   // dx = ax*dt^2
	abs_dist.y = ((abs_force.y * time) + bipedPhy01_surface_vel_vec[1]) * TIME;   // dy = ay*dt^2
//	abs_dist.z = (my._gid01_vel_z + bipedPhy01_surface_vel_vec[2])* TIME;       // dz = vz*dt
	abs_dist.z = (bipedPhy01_surface_vel_vec[2])* TIME;       // dz = vz*dt


	// 'normal' move
	if(my._move_min_z__003 >= 0) { move_min_z = min(0.99,my._move_min_z__003); }    // set the min_z glide value
/*
//Returned back to 1 move (new collision no longer needs two?)
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box;
	c_move(my,nullvector,abs_dist,move_mode); // abs distance move..
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide;
	result = c_move(my,dist,nullvector,move_mode); // ground (and air) move..
*/

	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide;
	result = c_move(my,dist,abs_dist,move_mode); // ground (and air) move..


	move_min_z = 0;


ifdef	BIPEDPHY01_DEBUG;
bipedPhy01_debug_data[12] = my_speed.x;
bipedPhy01_debug_data[13] = my_speed.y;
bipedPhy01_debug_data[14] = my_speed.z;
bipedPhy01_debug_data[15] = my._gid01_vel_x;
bipedPhy01_debug_data[16] = bipedPhy01_surface_vel_vec[2];
bipedPhy01_debug_data[17] = my._gid01_vel_z;
endif;

	// calculate distance for animation
	if(result > 0)
	{
		// only use the relative distance traveled (for animation)
		my._biped01_dist = vec_length(dist);
		my._biped01_dist = min(my._biped01_dist,result);
		if(force_vec[0] < 0) { my._biped01_dist *= -1; }	// walking backwards
	}
	else
	{
		// player is not moving, do not animate
		my._biped01_dist = 0;
	}

ifdef	BIPEDPHY01_DEBUG;
BipedPhy01_Vis_Col();
endif;
}




// =====================================================================
// UPDATE FUNCTION

// Desc: update a biped entity for a frame
//
// Assume: 'my' is set to a valid biped01 entity (see doc for valid skills)
function	BipedPhy01_Update(&force_vec, pan_dist)
{
	var	biped_content;	// 1-empty, 2-passable (water), 3-solid

ifdef	bipedPhy01_perform_report;
timer();
endif;
	// scan floor and check to see if we are in solid, passable, etc.
	// sets: bipedPhy01_height: height above nearest surface
	//		   bipedPhy01_surface_normal_vec[3]: normal vector of surface
	biped_content = BipedPhy01_Scan();
ifdef	bipedPhy01_perform_report;
bipedPhy01_scan_elapsed = timer();
endif;

ifdef	BIPEDPHY01_DEBUG;
bipedPhy01_debug_data[0] = bipedPhy01_height;
bipedPhy01_debug_data[1] = trace_hit;
bipedPhy01_debug_data[2] = my._biped01_state;
vec_set( bipedPhy01_debug_data[5],bipedPhy01_surface_normal_vec);
endif;

	// if we haven't set a state yet, set the state to walking (default state)
	if(my._biped01_state == biped01_state_none_const)
	{
		my._biped01_state = biped01_state_walk_const;
	}

ifdef	bipedPhy01_perform_report;
bipedPhy01_calls += 1;
bipedPhy01_move_calls += 1;
timer();
endif;
   BipedPhy01_Move_Ground(force_vec,pan_dist);//!!!!	BipedPhy01_Move_Ground(force_vec, pan_dist);
ifdef	bipedPhy01_perform_report;
bipedPhy01_move_elapsed = timer();
endif;

	//+++ clean up

}


ifelse;  // GID01_OLD_COLLIDE is SET

//==================================================================================
//==== Movement code using "OLD" collision (for speed) =============================
//==================================================================================


// Desc: handle biped panning with absolute values (not time correct)
//
// Input: ang_dist (vec) set to the panning amount (modified)
//
// Result:
//		Sets: my._gid01_ang_vel_pan equal to the angular velocity
//				ang_dist is set to the actual distace moved this frame
//		The entity 'my' is rotated using c_rotate().
// Assume: my is set to a biped01 entity,
function BipedPhy01_Pan(ang_dist)        // OLD
{
	my.pan += ang_dist;

	my._gid01_ang_vel_pan = ang_dist / TIME; // calculate ang vel for use later
}


// desc: simple scan. Does NOT detect passable blocks (water,forcefields, etc.)
//
// Sets:  bipedPhy01_height: distance (quants) above a solid object (may be negitive)
//			 bipedPhy01_surface_normal_vec: normal of the object scanned
//			 bipedPhy01_surface_vel_vec: speed of any platform scanned (nullvector if none)
//
// Assume: my is set to a valid biped01 entity (see doc for valid skills)
//			  Entity is 2000 units or LESS above a solid block/entity
var bipedPhy01_to_vec[3];   // can't use local vecs when calling functions
var bipedPhy01_from_vec[3];
function	BipedPhy01_Scan()  // OLD
{
if(my == NULL) { diag("************NULLED MY IN BipedPhy01_Scan****"); }

 	// Use trace to scan the surface under the entity
	vec_set(bipedPhy01_from_vec,my.x);
	vec_set(bipedPhy01_to_vec,my.x);
	trace_mode = IGNORE_ME + IGNORE_PASSABLE + IGNORE_PASSENTS + ACTIVATE_SONAR + USE_BOX;
	bipedPhy01_to_vec.z -= 2000;//-= (160 - my.min_z);//+= (my.min_z - 15); //-= 4000;
	bipedPhy01_height = trace(bipedPhy01_from_vec,bipedPhy01_to_vec);

	bipedPhy01_temp_ent = you;	// save entity hit...

	vec_set(bipedPhy01_surface_normal_vec.x,NORMAL.X); // normal is set in trace

	if(bipedPhy01_surface_normal_vec[2] < bipedPhy01_slope_fac)
	{
		// redo the trace without box (to adjust for slope)
		trace_mode = IGNORE_ME + IGNORE_PASSABLE + IGNORE_PASSENTS + ACTIVATE_SONAR;
		bipedPhy01_height = trace(bipedPhy01_from_vec,bipedPhy01_to_vec);
		bipedPhy01_height += my.min_z;     // adjust height since we don't use box
	}


	// if the player is standing on a platform or in a forcefield,
	if(bipedPhy01_temp_ent != NULL)
	{
		if(gid01_platform_const == bipedPhy01_temp_ent._gid01_id)
		{
			// add current platform velocity
			vec_set(bipedPhy01_surface_vel_vec[0],bipedPhy01_temp_ent._gid01_vel_x);
		}
		else
		{
			// no added velocity due to platform
			vec_set(bipedPhy01_surface_vel_vec[0],nullvector);
		}
	}
	else
	{
		// no added velocity
		vec_set(bipedPhy01_surface_vel_vec[0],nullvector);
	}

	return(0);
}



// Desc: move on the ground or air
//
// assume: my is set to a biped01
//			  bipedPhy01_height is set to the current height above a surface (may be negitive)
//			  bipedPhy01_surface_normal_vec: normal of the object scanned
//			  bipedPhy01_surface_vel_vec: speed of any platform scanned (nullvector if none)
//			  bipedPhy01_grnd_friction is set to the ground friction value
function	BipedPhy01_Move_Ground(&force_vec, pan_dist)  // OLD
{

	var	friction;	// friction constant
 //---	var	delta_fric; // friction over time value
	var	dist[3];		// distance covered this frame in direction faced
	var	abs_dist[3];// additional distance covered this frame in world space
	var	abs_force[3];//forces from the environment (gravity, elevators, etc.)
	var	q_jump_or_duck; // 1: jump, -1: duck, 0: neither
//---	var	old_loc[3];	//  old location

	//----------------------------------
	// Adjust input forces
	q_jump_or_duck = sign(force_vec[2]);
	force_vec[2] = 0;     // null out, jumping is handled here (no degree of jump/duck)


	//---------------------------------
	// rotate the player (absolute)
	BipedPhy01_Pan(pan_dist);
	// this biped doesn't tilt or roll
	my.tilt = 0;
	my.roll = 0;


   // Handle biped diferently if it is in the air or on the ground.
	if(bipedPhy01_height > 2) // in the air
	{
ifdef	BIPEDPHY01_DEBUG;
Draw_Text("AIR",50,80,vector(255,0,0));
endif;

		friction = bipedPhy01_air_friction; // use air friction

   	// airborne - reduce all relative forces
		// to prevent him from jumping or further moving in the air
		force_vec[0] *= 0.2; // don't set the force completely to zero, otherwise
		force_vec[1] *= 0.2; // player could be stuck on top of a non-wmb entity
		force_vec[2] = 0;

		// gravity pulls on biped..
		vec_set(abs_force,vector(0,0,-bipedPhy01_gravity));


 		// only falling if moving downward (not if jumping up)
		if( (my._gid01_vel_z < 0))
		{
			// check to make sure we are not in another state
			if(my._biped01_state_mod != biped01_state_mod_fall_const)
			{
				my._biped01_state_mod = biped01_state_mod_fall_const; // set falling flag
				my._biped01_state_mod_value = 0; // reset falling time
			}
			my._biped01_state_mod_value += TIME;   // add falling time
		}


	}
	else    // on/in the Ground
	{
		friction = bipedPhy01_grnd_friction; // use ground friction

		//-------------------------------------------------------------------------------------------------------------------------
		// Calculate falling damage
 		if((my._biped01_state_mod == biped01_state_mod_fall_const) &&  // we are falling
			(my._biped01_state_mod_value > bipedPhy01_safe_fall_time_const))	// we've been falling more then the safe amount of time
  		{
			// indicate damage from environment
			my._biped01_state2_mod = biped01_state_mod_damage_const;
			my._biped01_state2_mod_value = BipedPhy01_Fall_Damage();		// take damage depending on fall_time
			my._biped01_state_mod = biped01_state_mod_none_const; // reset flag (we are on ground)
			my._biped01_state_mod_value = 0;
		}
		else
		{
			// note: this works ONLY because no other state can exist at this point
			//(e.g. the player is not underwater and not falling)
			my._biped01_state_mod = biped01_state_mod_none_const; // reset flag (we are on ground)
			my._biped01_state_mod_value = 0;
		}
		//-------------------------------------------------------------------------------------------------------------------------

 		// reset absolute forces
		vec_set(abs_force,nullvector);




		// do we wish to jump?
		if((q_jump_or_duck>0) && (my._biped01_jump_target <= 0)    // no jumping from ducking, or if we are already jumping
			&& (bipedPhy01_surface_normal_vec.z >= bipedPhy01_slope_fac) && (my._biped01_state != biped01_state_wade_const)) // no jumping from slopes or water
		{
			// start a jump
			my._biped01_jump_target = my._jump_height__003; // jump delta
			my._biped01_state = biped01_state_jump_const; // set state to jump
		}
		else
		{
			// walking/standing/running on solid ground
			my._biped01_state = biped01_state_walk_const;
		}


	} // end "on the ground"

	// Handle Slopes
	if(bipedPhy01_surface_normal_vec[2] < bipedPhy01_slope_fac)
	{
ifdef	BIPEDPHY01_DEBUG;
Draw_Text("SLOPE",50,120,vector(255,0,0));
endif;

		if(bipedPhy01_surface_normal_vec[2] == 0) // happens if biped is on a model entity
		{
			abs_force[0] = bipedPhy01_gravity * bipedPhy01_surface_normal_vec[0];
			abs_force[1] = bipedPhy01_gravity * bipedPhy01_surface_normal_vec[1];
			abs_force[2] = -bipedPhy01_gravity;
		}
		else
		{
			// vector pointing down the slope
 			abs_force[0] = bipedPhy01_surface_normal_vec[2] * bipedPhy01_surface_normal_vec[0];
			abs_force[1] = bipedPhy01_surface_normal_vec[2] * bipedPhy01_surface_normal_vec[1];
			abs_force[2] = bipedPhy01_surface_normal_vec[2] * bipedPhy01_surface_normal_vec[2] - 1;
 			vec_scale(abs_force,bipedPhy01_gravity);
		}
	}


	// calculate velocities
	dist.x = accelerate(my._gid01_vel_x,force_vec[0],friction);
	dist.y = accelerate(my._gid01_vel_y,force_vec[1],friction);
	dist.z = 0;                      // dz = 0  (only gravity and jumping)


	if(my._biped01_jump_target <= 0)	// not jumping
	{
		// push out of ground
		if(bipedPhy01_height < 0)
		{
			abs_dist.z = -bipedPhy01_height;          // "pop" out of ground
			my._gid01_vel_z = max(0,my._gid01_vel_z);	// stop downward forces
		}
		else
		{
		 	//	my._gid01_vel_z = (TIME * abs_force.z) + (delta_fric * my._gid01_vel_z);    // gravity force (old way)
			abs_dist.z = accelerate(my._gid01_vel_z,abs_force.z,friction);	// new way
		}
	}
	else	// jumping
	{
	 	my._gid01_vel_z = sqrt(my._biped01_jump_target*2*bipedPhy01_gravity);    // jumping force
		my._biped01_jump_target -= my._gid01_vel_z * TIME;	// reduce the delta
		abs_dist.z = my._gid01_vel_z * TIME;
	}

	// calculate absolute distance to move
	// NOTE:
	//  - bipedPhy01_surface_vel_vec is any velocity added from standing on a moving platform
	abs_dist.x = ((abs_force.x * time) + bipedPhy01_surface_vel_vec[0]) * TIME;   // dx = ax*dt^2
	abs_dist.y = ((abs_force.y * time) + bipedPhy01_surface_vel_vec[1]) * TIME;   // dy = ay*dt^2
//	abs_dist.z = (my._gid01_vel_z + bipedPhy01_surface_vel_vec[2])* TIME;       // dz = vz*dt
	abs_dist.z += (bipedPhy01_surface_vel_vec[2])* TIME;       // dz = vz*dt


	// 'normal' move
//	if(my._move_min_z__003 >= 0) { move_min_z = min(0.99,my._move_min_z__003); }    // set the min_z glide value

	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide;
	result = ent_move(dist,abs_dist); // ground (and air) move..


//	move_min_z = 0;


ifdef	BIPEDPHY01_DEBUG;
bipedPhy01_debug_data[12] = my_speed.x;
bipedPhy01_debug_data[13] = my_speed.y;
bipedPhy01_debug_data[14] = my_speed.z;
bipedPhy01_debug_data[15] = my._gid01_vel_x;
bipedPhy01_debug_data[16] = my._gid01_vel_y;
bipedPhy01_debug_data[17] = my._gid01_vel_z;
endif;

	// calculate distance for animation
	if(result > 0)
	{
		// only use the relative distance traveled (for animation)
		my._biped01_dist = vec_length(dist);
		my._biped01_dist = min(my._biped01_dist,result);
		if(force_vec[0] < 0) { my._biped01_dist *= -1; }	// walking backwards
	}
	else
	{
		// player is not moving, do not animate
		my._biped01_dist = 0;
	}
}



// =====================================================================
// UPDATE FUNCTION

// Desc: update a biped entity for a frame
//
// Assume: 'my' is set to a valid biped01 entity (see doc for valid skills)
function	BipedPhy01_Update(&force_vec, pan_dist)   // OLD
{
	var	biped_content;	// 1-empty, 2-passable (water), 3-solid

ifdef	bipedPhy01_perform_report;
timer();
endif;
	// scan floor and check to see if we are in solid, passable, etc.
	// sets: bipedPhy01_height: height above nearest surface
	//		   bipedPhy01_surface_normal_vec[3]: normal vector of surface
	biped_content = BipedPhy01_Scan();
ifdef	bipedPhy01_perform_report;
bipedPhy01_scan_elapsed = timer();
endif;

ifdef	BIPEDPHY01_DEBUG;
bipedPhy01_debug_data[0] = bipedPhy01_height;
bipedPhy01_debug_data[1] = trace_hit;
bipedPhy01_debug_data[2] = my._biped01_state;
vec_set( bipedPhy01_debug_data[5],bipedPhy01_surface_normal_vec);
endif;

	// if we haven't set a state yet, set the state to walking (default state)
	if(my._biped01_state == biped01_state_none_const)
	{
		my._biped01_state = biped01_state_walk_const;
	}

ifdef	bipedPhy01_perform_report;
bipedPhy01_calls += 1;
bipedPhy01_move_calls += 1;
timer();
endif;
   BipedPhy01_Move_Ground(force_vec,pan_dist);
ifdef	bipedPhy01_perform_report;
bipedPhy01_move_elapsed = timer();
endif;

	//+++ clean up

}

endif;  // GID01_OLD_COLLIDE


//==================================================================================
//==================================================================================
//==================================================================================

ifdef	BIPEDPHY01_DEBUG;
include <debug01.wdl>;


// Desc: Draw a collision sphere used in c_move
//
// Note: This function only handles non-rotated bounding spheres.
//			It is only used as a rough estimate of the actual bounding sphere.
//
//	Note2: This is an "ugly hack" and is ment for debugging only!
var	debug01_old_pt[3];
var	debug01_new_pt[3];
var	debug01_old_pt2[3]; 	// mirror points
var	debug01_new_pt2[3];  // mirror points
function	BipedPhy01_Vis_Col()
{
	var	pt[3];
	var	c_pt[3];
	var	const[3];	//
	var	off_c_pt[3];
	var	old_x;
	var	old_y;

	// calculate center point
	c_pt.x = (my.max_x + my.min_x)/2;
	c_pt.y = (my.max_y + my.min_y)/2;
	c_pt.z = (my.max_z + my.min_z)/2;

	// calculate the offset centerpoint using rotation
	vec_set(off_c_pt,c_pt);
	vec_rotate(off_c_pt,my.pan);

	const.x = my.max_x - c_pt.x;

	Debug01_Draw_Line(vector(my.x+off_c_pt.x+const.x,my.y+off_c_pt.y,my.z+off_c_pt.z),
							vector(my.x+off_c_pt.x-const.x,my.y+off_c_pt.y,my.z+off_c_pt.z));

	const.x *= const.x;
	const.y = my.max_y - c_pt.y;

	Debug01_Draw_Line(vector(my.x+off_c_pt.x,my.y+off_c_pt.y+const.y,my.z+off_c_pt.z),
							vector(my.x+off_c_pt.x,my.y+off_c_pt.y-const.y,my.z+off_c_pt.z));

	const.y *= const.y;
	const.z = my.max_z - c_pt.z;
	const.z *= const.z;



	// ==========================================
	// Front/Back
	vec_set(debug01_new_pt,off_c_pt);
	vec_add(debug01_new_pt,my.x);
	vec_set(debug01_new_pt2,debug01_new_pt);
	pt.y = 0;
	pt.z = my.max_z - c_pt.z;
	while(pt.z > -(my.max_z - c_pt.z))
	{
		old_x = pt.x;
		pt.x = sqrt( (const.x*(1- ( (pt.z*pt.z)/(const.z) ) )) );

		vec_set(debug01_old_pt,debug01_new_pt);// save old
		vec_set(debug01_new_pt,pt);            // create new
		vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
		// use calculations to find back arc
		vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
		vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
		vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
		vec_add(debug01_new_pt,my.x);          // offset by player position
		vec_add(debug01_new_pt2,my.x);         // offset by player position
		vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
		vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
		Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
		Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc

		pt.z -= 5;
	}
 	old_x = pt.x;
	pt.z = -(my.max_z - c_pt.z);
 	pt.x = sqrt( (const.x*(1- ( (pt.z*pt.z)/(const.z) ) )) );

	vec_set(debug01_old_pt,debug01_new_pt);// save old
	vec_set(debug01_new_pt,pt);            // create new
	vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
	// use calculations to find back arc
	vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
	vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
	vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
	vec_add(debug01_new_pt,my.x);          // offset by player position
	vec_add(debug01_new_pt2,my.x);         // offset by player position
	vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
	vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
	Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
	Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc

	// =====================================
	// Left/Right
	pt.x = 0;
	pt.z = my.max_z - c_pt.z;
	while(pt.z > -(my.max_z - c_pt.z))
	{
		old_y = pt.y;
		pt.y = sqrt( (const.y*(1- ( (pt.z*pt.z)/(const.z) ) )) );

		vec_set(debug01_old_pt,debug01_new_pt);// save old
		vec_set(debug01_new_pt,pt);            // create new
		vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
		// use calculations to find back arc
		vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
		vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
		vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
		vec_add(debug01_new_pt,my.x);          // offset by player position
		vec_add(debug01_new_pt2,my.x);         // offset by player position
		vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
		vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
		Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
		Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc
		pt.z -= 5;

	}
 	old_y = pt.y;
	pt.z = -(my.max_z - c_pt.z);
 	pt.y = sqrt( (const.y*(1- ( (pt.z*pt.z)/(const.z) ) )) );

	vec_set(debug01_old_pt,debug01_new_pt);// save old
	vec_set(debug01_new_pt,pt);            // create new
	vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
	// use calculations to find back arc
	vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
	vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
	vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
	vec_add(debug01_new_pt,my.x);          // offset by player position
	vec_add(debug01_new_pt2,my.x);         // offset by player position
	vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
	vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
	Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
	Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc

	// scan trace line
	Debug01_Draw_Line(my.x,vector(my.x,my.y,my.z+(2*my.min_z-6)));


	my.transparent = on;


}
endif;   // BIPEDPHY01_DEBUG


endif;// bipedPhy01_wdl