ifndef bipedSnd_wdl;
define bipedSnd_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
//
// title: Biped Sounds (01)
// class: SOUND
// type: SUPPORT
// image: bipedSnd01.pcx
// help: Handle the playing of sounds for biped models.
//
// needs: bipedSnd01.wdl
// uses:
//
// prefix: bipedSnd01_
// idcode: 027
//
// ENDHEADER
// ---------------------------------------------------------------------




// ---------------------------------------------------------------------
// section: Walking Sound

// entry: Sound File
// help: Walk on ground (normal 'step') noise
// id: 1
sound		bipedSnd01_step1_snd1 = <step.wav>;
// entry: Cycle Distance
// help: Distance between step noises (in quants)
// id: 2
define	bipedSnd01_step1_dist1 = 75;
// entry: Volume
// help: Volume of step sound noise
// id: 3
define	bipedSnd01_step1_vol1 = 75;

// ---------------------------------------------------------------------
// section: Swimming Sound

// entry: Sound File
// help: Water noise (used for swimming/wading)
// id: 4
sound		bipedSnd01_swim_snd1 = <swim.wav>;
// entry: Cycle Distance
// help: Distance between swimming noise
// id: 5
define	bipedSnd01_swim_dist1 = 50;
// entry: Volume
// help: Volume of swimming noise
// id: 6
define	bipedSnd01_swim_vol1 = 50;

// ---------------------------------------------------------------------
// section: Damage Sound

// entry: Sound File
// entry: Damage noise 1
//id: 7
sound		bipedSnd01_damage1_snd1 = <damage1.wav>;


//++++++++
// ---------------------------------------------------------------------
// Sound files (biped 2)

//

// ---------------------------------------------------------------------
// Sound files (biped 3)

//


// Desc: Play sounds depending on the movement and state of the biped.
//
//   'my' must be set before calling
//		my._biped01_dist must be set to the distance that the actor has covered (in quants)
function BipedSnd01_Update_Loop()
{
	proc_late();	// this function relies on values being set in other functions

	var	dist;	// total distance traveled since last sound (step, swim, etc).

	while(my != NULL)
	{


		// if NOT dead
		if((my._biped01_state != biped01_state_dying_const) || (my._biped01_state != biped01_state_dead_const))
		{
			// Get biped state
			// walking, running, or standing mode
			if(my._biped01_state == biped01_state_walk_const)
			{
				// walking/running sound
				dist += abs(my._biped01_dist);
				while(dist > bipedSnd01_step1_dist1)
				{
					// ++ determine which sound to play..++

					// play walking sound
					ent_playsound(my,bipedSnd01_step1_snd1,bipedSnd01_step1_vol1);

					// reset value
					dist -= bipedSnd01_step1_dist1;
				}
			}
			else
			{
				if((MY._biped01_state == biped01_state_wade_const) || (MY._biped01_state == biped01_state_swim_const))
				{
					// swimming/wading sound
					dist += abs(my._biped01_dist);
					while(dist > bipedSnd01_swim_dist1)
					{
						// ++ determine which sound to play..++

						// play water sound
						ent_playsound(my,bipedSnd01_swim_snd1,bipedSnd01_swim_vol1);

						// reset value
						dist -= bipedSnd01_swim_dist1;
					}
				}

			}
		}// end if NOT dead





		// once off sounds
//++biped01_state_dying_const
//++biped01_state_jump_const
		// play once on trigger



		wait(1); // do every frame
	}

}


endif; // bipedSnd01_wdl