ifndef camera1stPerson_wdl;
define camera1stPerson_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
//
// title: First Person Camera
// class: CAMERA
// type: USER
// image: camera1stPerson01.pcx
// help: The first person camera is an "in the head" view.
// help: Standard FPS type camera.
//
//
// uses: display_active_camera_vec
// needs: display00.wdl
// needs: cameraSelect.wdl cameraTarget.wdl
//
//
// prefix: camera1stPerson01_
// idcode: 004
//
// ENDHEADER
// ---------------------------------------------------------------------




// General Camera Entries


// entry: ID number
// help: Identification number for this camera (used in cameraSelect)
// help: NOTE: Action not defined if two camera share a single value
// cntl: spin 1 x 1
// id: 2
define	camera1stPerson01_id = 1;

// enable: Hide Focus
// help: Hide the player model when camera is inside of it? (1-yes, 0-no)
// id: 1
define	camera1stPerson01_hide_focus_b;


// enable: Selectable?
// help: Make camera selectable by user? (1-yes, 0-no)
// id: 3
define	camera1stPerson01_user_sel_b;


// enable: Avatar view?
// help: Is this an "avatar" camera?
// help: Avatar cameras are cameras who's source/pan are those of the player's eyes
// id: 4
define	camera1stPerson01_avatar_q;



// Local variables
var	camera1stPerson01_pos_vec[3];	// x,y,z postion of the camera
var	camera1stPerson01_ang_vec[3];	// pan,tilt,roll angles of the camera
var	camera1stPerson01_bit_check; 	// used in bit-compairs to check is this camera is active


/////////////////////////////////////////////////////////////////////////
// desc: Used to attach the camera to an entity
//
starter Camera1stPerson01_Init
{
	var	updated_b;	// has the camera been update yet this frame?
	var	view_number;	// the current view being concidered
	diag("\nWDL-Loaded:cameraFirstPerson01.wdl");


	// init bit-field value used to check to see if this camera is active
	camera1stPerson01_bit_check = 1<<camera1stPerson01_id;

	ifdef camera1stPerson01_user_sel_b;
//	if(1 == camera1stPerson01_user_sel_b)
//	{
		// increase the camera select "max_camera" value if this value fall outside the current range
		if(camera1stPerson01_bit_check > cameraSelect_max_camera)
		{ cameraSelect_max_camera = camera1stPerson01_bit_check; }

		// add this camera type to the collection of valid cameras in camera select
		cameraSelect_valid_cameras |= camera1stPerson01_bit_check;
//	}
	endif;


	while(1)
	{
		updated_b = 0;	// camera hasn't been updated yet this frame
		view_number = 0;	// start with main view

		// for all active views
		while(view_number < display_active_views)
		{
			// check to see if this camera updates the current view
			if(display_active_camera_vec[view_number] & camera1stPerson01_bit_check)
			{
				// if the camera hasn't updated itself this frame...
				if(0 == updated_b)
				{
					// update camera position
					Camera1stPerson01_Update();

					// camera has been updated this frame
					updated_b = 1;
				}

				// update current view
				Camera1stPerson01_Update_View(view_number);

			}

			view_number +=1;
		}

		wait(1);	// every frame
	}

}

// desc: update the camera
function Camera1stPerson01_Update()
{
	proc_late();
	// set the camera's position
	vec_set(camera1stPerson01_pos_vec,cameraTarget_pos_vec);
	vec_set(camera1stPerson01_ang_vec,cameraTarget_ang_vec);
}


// desc: update the view whos ID is passed in to the position/orientation
//      of the Camera
function	Camera1stPerson01_Update_View(view_number)
{
	if(view_number == 0)
	{
		// update main view (camera)
		vec_set(camera.x,camera1stPerson01_pos_vec.x);
		vec_set(camera.pan,camera1stPerson01_ang_vec.pan);

		ifdef camera1stPerson01_hide_focus_b;
		//if(1==camera1stPerson01_hide_focus_b)
		//{	// hide the target entity from view if inside
  			//camera.genius = cameraTarget_ent;  // camera.genius does not work with new collision
			if(cameraTarget_ent != NULL) { cameraTarget_ent.invisible = on; }
		//}
		ifelse;
		//else
		//{	// show the target entity no matter what
			camera.genius = 0;
		//}
		endif;

		// is this an avatar view?
		ifdef camera1stPerson01_avatar_q;
				cameraTarget_avatar_view_q = 1;
		ifelse;
				cameraTarget_avatar_view_q = 0;
		endif;
	}

	// +++ other views


}

endif;