ifndef plSelect_wdl;
define plSelect_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Player Select
// class: PLAYER
// type: SUPPORT
// image: plSelect.pcx
// help: Master selection function. Like cameraSelect, allows the user
// help:control to select from a list of possible player models (biped, car,
// help:spaceship, airplane, etc.).
// help:
// help: Also stores values that are shared ammong player models.
// help:	Note: "model" in this function refers to the movement/input model used,
// help:not the "mdl" itself.
//
//
// prefix: plSelect_
// idcode: 017
//
// needs: cameraTarget.wdl
//
// uses:  cameraTarget::cameraTarget_ang_vec, cameraTarget_pos_vec
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// LOCAL VALUES

// pointer to the current player entity (0 if not valid)
entity* plSelect_curr_ent;

// current view type

// current player model (enum)
var	plSelect_curr_model = 0;

// Bitfield used to track the number of valid player models
var	plSelect_valid_models = 0;

// Largest bit value for a valid player model
var 	plSelect_max_model = 0;

// General Player State
var	plSelect_state = 0;
define	plSelect_state_unknown = 0;// player state is not defined (not yet loaded?)
define	plSelect_state_active = 1;	// player model is active
define	plSelect_state_dead = 2;   // player model is dead
define	plSelect_state_paused = 3;	// player is paused


// Local Functin Prototypes
function	PlSelect_Switch(modelID);	// switch to a player model


//Desc: Init player select
starter PlSelect_Init()
{
// entry: ID
// help: ID of starting player model (-1 to auto select on start)
// cntl: spin -1 x 1
// id: 1
	var	auto_select_const = -1;

	// wait for at least one model to become valid
	while(plSelect_valid_models == 0)
	{
		wait(1);
	}

	wait(2);	// wait to make sure that all models have been loaded
	diag("\nWDL-Loaded:plSelect.wdl");

	if(auto_select_const < 0)
	{
		// select the first model from the list of valid models
		plSelect_curr_model = 0;
		temp = 0;
		while((plSelect_curr_model < plSelect_max_model) &&
			( ((temp & plSelect_valid_models) == 0) ))
		{
			plSelect_curr_model += 1;
			temp = 1<<plSelect_curr_model;
		}
	}
	else
	{
		plSelect_curr_model = auto_select_const;
	}

}

//Desc: Switch to new player model   ++++++
function	PlSelect_Switch(modelID)
{
	plSelect_curr_model = modelID;
}


endif;